﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Runtime.InteropServices.WindowsRuntime;
using Windows.Devices.Enumeration;
using Windows.Devices.I2c;
using Windows.Foundation;
using Windows.Foundation.Collections;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Controls.Primitives;
using Windows.UI.Xaml.Data;
using Windows.UI.Xaml.Input;
using Windows.UI.Xaml.Media;
using Windows.UI.Xaml.Navigation;

// 空白ページのアイテム テンプレートについては、http://go.microsoft.com/fwlink/?LinkId=402352&clcid=0x409 を参照してください

namespace Tempapp01
{
    /// <summary>
    /// ADT7410 I2C Temperature Sensor App
    /// </summary>
    public sealed partial class MainPage : Page
    {
        private const string I2C_CONTROLLER_NAME = "I2C1";
        private const byte   TEMPIC_ADT7410_I2C_ADDR = 0x48; /* ADT7410 default addres */
        private I2cDevice TEMPIC_ADT7410;

        private const byte TEMPIC_ADT7410_REG_MSB    = 0x00;  /* offset 0 : MSB of temp */
        private const byte TEMPIC_ADT7410_REG_LSB    = 0x01;  /* offset 1 : LSB of temp */
        private const byte TEMPIC_ADT7410_REG_STATUS = 0x02;  /* offset 2 : status register */
        private const byte TEMPIC_ADT7410_REG_CONFIG = 0x03;  /* offset 3 : configuration register */

        private DispatcherTimer periodicTimer;

        private double currentTemp;

        private async void InitI2CTemp()
        {
            string aqs = I2cDevice.GetDeviceSelector();
            var dis = await DeviceInformation.FindAllAsync(aqs);
            if (dis.Count == 0)
            {
                Text_Status.Text = "No I2C controllers were found on the system";
                return;
            }
            var settings = new I2cConnectionSettings(TEMPIC_ADT7410_I2C_ADDR);
            settings.BusSpeed = I2cBusSpeed.FastMode;
            TEMPIC_ADT7410 = await I2cDevice.FromIdAsync(dis[0].Id, settings);    /* Create an I2cDevice with our selected bus controller and I2C settings */
            if (TEMPIC_ADT7410 == null)
            {
                Text_Status.Text = string.Format(
                    "Slave address {0} on I2C Controller {1} is currently in use by " +
                    "another application. Please ensure that no other applications are using I2C.",
                    settings.SlaveAddress,
                    dis[0].Id);
                return;
            }

            /* set A/D resolution. 0x80 -> 16bit mode, 0x00 -> 13bit mode  */
            byte[] WriteBuf_SetMode = new byte[] { TEMPIC_ADT7410_REG_CONFIG, 0x80 };    /* set 16bit mode */
            try
            {
                TEMPIC_ADT7410.Write(WriteBuf_SetMode);
            }
            catch (Exception ex)
            {
                Text_Status.Text = "Cannot configuration service: " + ex.Message;
                return;
            }

            periodicTimer = new DispatcherTimer();
            periodicTimer.Interval = TimeSpan.FromMilliseconds(1000);
            periodicTimer.Tick += Periodic_MeasureTemp;
            periodicTimer.Start();
        }

        static int samplingCount = 1;
        private void Periodic_MeasureTemp(object sender, object e)
        {
            currentTemp = MeasurementTemp();
            TempBox.Text = String.Format(" {0}°C", currentTemp);

            Text_Status.Text = "Status: Running (sampling count = " + samplingCount + ")";
            samplingCount++;
        }

        private double MeasurementTemp()
        {
            byte[] RegAddrBuf = new byte[] { TEMPIC_ADT7410_REG_MSB }; /* Register address to read */
            byte[] ReadBuf    = new byte[2];  /* 2 byte value */
            TEMPIC_ADT7410.WriteRead(RegAddrBuf, ReadBuf);

            // result is 16bit integer
            // bit 15 = sign bit. 0 -> zero or positive temp, 1 -> minus temp.
            // bit 14..7 = integer part.
            // bit 6..0  = fraction part.
            double result;
            if ((ReadBuf[0] & 0x80) == 0)
            {
                result = (ReadBuf[0] * 256.0 + ReadBuf[1]) / 128.0;
            }
            else
            {
                result = (ReadBuf[0] * 256.0 + ReadBuf[1] - 65536.0) / 128.0;
            }
            result = Math.Round(result, 3);
            return result;
        }

        public MainPage()
        {
            this.InitializeComponent();

            InitI2CTemp();
        }
    }
}
